package de.tu_dresden.lat.dlProofChecking

import java.io.File

import de.tu_dresden.lat.dlProofChecking.proofsProcessing.{CertificateGenerator, ClassificationCounter}
import de.tu_dresden.lat.proofs.ELKProofGenerator
import org.semanticweb.elk.owlapi.ElkReasonerFactory
import org.semanticweb.owlapi.apibinding.OWLManager
import org.semanticweb.owlapi.model.parameters.Imports
import org.semanticweb.owlapi.model._

import scala.collection.JavaConverters._

object GenerateClassificationCertificate {
  def main(args: Array[String]) = {

    val manager = OWLManager.createOWLOntologyManager()

    val file = //if(args.isEmpty)
//      new File("/Users/koopmann/Documents/Ontologies/pool_sample/el/classification/ore_ont_9499.owl")
 //   else
      new File(args(0))

    val onlyCount = args.size>1 && args(1).equals("onlyCount")

    if(onlyCount)
      println("only counting!")

    if(!file.exists()) {
      println("Expect existing folder or file name as argument")
      System.exit(1)
    }



    val reasonerFactory = new ElkReasonerFactory()


    val proofGenerator = new ELKProofGenerator()
    //new ProofWithJustificationGenerator(new ELKProofGenerator())c
    //proofGenerator.setReasonerFactory(reasonerFactory)

    val certificateGenerator =
      new CertificateGenerator(reasonerFactory, proofGenerator)


    val classificationCounter = new ClassificationCounter(reasonerFactory, proofGenerator)

    val files =
      if(file.isDirectory)
        file.listFiles()
          .sortBy(_.length)
          .filter(_.getName.endsWith(".owl"))
          .foreach(file =>
            if(onlyCount)
              classificationCounter.processOntology(manager.loadOntologyFromOntologyDocument(file))
            else
              processFile(file, certificateGenerator, manager))
      else {
        if(onlyCount) {
          val ont = manager.loadOntologyFromOntologyDocument(file)
          ont.remove(ont.aboxAxioms(Imports.INCLUDED)) // no support for ABox yet

          println("axiomCount: "+ont.getAxiomCount(Imports.INCLUDED))
          classificationCounter.processOntology(ont)
        } else
          processFile(file, certificateGenerator, manager)
      }

  }

  def processFile(file: File, certificateGenerator: CertificateGenerator, manager: OWLOntologyManager){

      println()

      println(file)

      val ontology: OWLOntology = manager.loadOntologyFromOntologyDocument(file)

      ontology.remove(ontology.aboxAxioms(Imports.INCLUDED)) // no support for ABox yet

      ontology.axioms(Imports.INCLUDED).forEach{ax =>
        ax match {
          case shortCut: OWLSubClassOfAxiomSetShortCut => {
            ontology.remove(ax);
            ontology.add(shortCut.asOWLSubClassOfAxioms())
          }
          case _ => ; // other cases are fine
        }
        if(ax.getNestedClassExpressions.asScala.exists(_.isInstanceOf[OWLObjectOneOf]))
          ontology.remove(ax)
      }

      println(ontology.getAxiomCount(Imports.INCLUDED))

      println("Generating certificate!")

      certificateGenerator.generateCertificate(ontology,
        new File(file.getName().replace(".owl", ".plf")))


  }
}
