package de.tu_dresden.lat.dlProofChecking

import java.io.File

import de.tu_dresden.lat.dlProofChecking.proofsProcessing.{CertificateGenerator, ClassificationCounter}
import de.tu_dresden.lat.proofs.ELKProofGenerator
import org.semanticweb.elk.owlapi.ElkReasonerFactory
import org.semanticweb.owlapi.apibinding.OWLManager
import org.semanticweb.owlapi.model.parameters.Imports
import org.semanticweb.owlapi.model._

import scala.collection.JavaConverters._

object CheckSupported {
  def main(args: Array[String]) = {

    val file = new File(args(0))

    val manager = OWLManager.createOWLOntologyManager()

    val ont = manager.loadOntologyFromOntologyDocument(file)
    ont.remove(ont.aboxAxioms(Imports.INCLUDED)) 

    ont.logicalAxioms().forEach{
      _ match {
	case _: OWLSubClassOfAxiom => ; // okay
	case _: OWLEquivalentClassesAxiom => ; // okay
	case _: OWLSubObjectPropertyOfAxiom => ; // okay
	case _: OWLSubPropertyChainOfAxiom => ; // okay
	case _: OWLObjectPropertyDomainAxiom => ; // okay
	case _: OWLTransitiveObjectPropertyAxiom => ; // okay
	case other => fail(file+" is not supported, as it contains: "+other)
      }
    }

    ont.getNestedClassExpressions().forEach{
      _ match {
	case _: OWLObjectIntersectionOf => ; // okay
	case _: OWLObjectSomeValuesFrom => ; // okay
	case _: OWLClass => ; // okay
	case other => fail(file+" is not supported, as it contains: "+other)
      }
    }

    println(file+" is supported.")
  }

  def fail(message: String) = {
    println(message)
    System.exit(0)
  }
	    
}
