package de.tu_dresden.lat.dlProofChecking.lispFormatting

import org.semanticweb.owlapi.model.{OWLAxiomVisitorEx, OWLDataFactory, OWLDisjointClassesAxiom, OWLEquivalentClassesAxiom, OWLObjectPropertyDomainAxiom, OWLObjectPropertyExpression, OWLObjectPropertyRangeAxiom, OWLSubClassOfAxiom, OWLSubObjectPropertyOfAxiom, OWLSubPropertyChainOfAxiom, OWLTransitiveObjectPropertyAxiom}

import scala.collection.JavaConverters._

class ELAxiom2LISPFormatter(factory: OWLDataFactory) extends OWLAxiomVisitorEx[String] {

  val conceptFormatter = new ELClass2LISPFormatter(factory)
  val roleFormatter = ELObjectProperty2LISPFormatter

  override def doDefault[T](obj: T): String =
    throw new UnsupportedOperationException("Not supported: "+obj)

  override def visit(gci: OWLSubClassOfAxiom) = {
    val subC = gci.getSubClass.accept(conceptFormatter)
    val superC = gci.getSuperClass.accept(conceptFormatter)
    assert(subC!= null, "subclass not supported: "+gci)
    assert(superC!= null, "superclass not supported: "+gci)
    "(sub "+subC+" "+superC+")"
  }

  override def visit(eq: OWLEquivalentClassesAxiom) = {
    "(eq "+eq.classExpressions().iterator.asScala.map(d => d.accept(conceptFormatter)).mkString(" ")+")"
  }

  override def visit(ri: OWLSubObjectPropertyOfAxiom) = {
    "(rsub "+conceptFormatter.formatRole(ri.getSubProperty)+" "+conceptFormatter.formatRole(ri.getSuperProperty())+")"
  }

  override def visit(ri: OWLSubPropertyChainOfAxiom) = {
    "(rsub "+formatChain(ri.getPropertyChain.asScala.toList)+" "+conceptFormatter.formatRole(ri.getSuperProperty())+")"
  }

  override def visit(ax: OWLObjectPropertyDomainAxiom) = {
    "(roleDomain "+conceptFormatter.formatRole(ax.getProperty)+" "+ax.getDomain.accept(conceptFormatter)+")"
  }

  override def visit(ax: OWLObjectPropertyRangeAxiom) = {
    "(roleRange "+conceptFormatter.formatRole(ax.getProperty)+" "+ax.getRange.accept(conceptFormatter)+")"
  }

  override def visit(ax: OWLTransitiveObjectPropertyAxiom) = {
    "(transRole "+conceptFormatter.formatRole(ax.getProperty)+")"
  }

  def formatChain(roleChain: List[OWLObjectPropertyExpression]): String = roleChain match {
    case Nil => "erc"
    case head::tail =>
      "(rc "+
        conceptFormatter.formatRoleName(head.getNamedProperty) +
        " "+
        formatChain(tail) +
        ")"
  }
}
