package de.tu_dresden.lat.dlProofChecking.lispFormatting

import java.util

import de.tu_dresden.lat.dlProofChecking.tools.StringNormaliser
import org.semanticweb.owlapi.model._

class ELClass2LISPFormatter(factory: OWLDataFactory) extends OWLClassExpressionVisitorEx[String]{

  val roleVisitor = ELObjectProperty2LISPFormatter

  override def doDefault[T](obj: T): String =
    throw new UnsupportedOperationException("class not supported: "+obj)

  override def visit(cl: OWLClass) =
    if(cl.isBottomEntity)
      "bot"
    else if (cl.isTopEntity)
      "top"
    else
    formatConceptName(cl)


  override def visit(cl: OWLObjectIntersectionOf) = {
    val conjuncts: util.List[OWLClassExpression] = cl.getOperandsAsList()
    if(conjuncts.isEmpty())
      factory.getOWLNothing.accept(this)
    if(conjuncts.size()==1)
      conjuncts.iterator().next().accept(this)
    else //if(conjuncts.size()>1)
      "(inter "+conjuncts.get(0).accept(this)+" "+
        factory.getOWLObjectIntersectionOf(conjuncts.subList(1,conjuncts.size())).accept(this)+ ")"

  }

  override def visit(cl: OWLObjectSomeValuesFrom) = {
    val role: OWLObjectPropertyExpression = cl.getProperty
    val filler = cl.getFiller
    "(ex "+formatRole(role)+" "+filler.accept(this)+")";
  }

  override def visit(cl: OWLObjectComplementOf) = {
    "(neg "+cl.getOperand.accept(this)+")"
  }

  def formatConceptName(cl: OWLClass) =
  "c"+formatIRI(cl.getIRI)

  def formatRole(role: OWLObjectPropertyExpression) = role match {
    case roleName: OWLObjectProperty => "(rc "+formatRoleName(roleName)+" erc)"
  }

  def formatRoleName(role: OWLObjectProperty) =
    "r"+formatIRI(role.getIRI)

  private def formatIRI(iri: IRI) =
    "<"+StringNormaliser.normalise(iri.toString)+">"
}
