package de.tu_dresden.lat.dlProofChecking.lispFormatting

import org.semanticweb.owlapi.model.{OWLAxiom, OWLClass, OWLEquivalentClassesAxiom, OWLObjectIntersectionOf, OWLObjectProperty, OWLObjectSomeValuesFrom, OWLOntology, OWLSubClassOfAxiom, OWLSubObjectPropertyOfAxiom}
import java.io.PrintWriter
import java.util
import java.util.Date

import de.tu_dresden.lat.dlProofChecking.tools.{ProofTools, StringNormaliser}
import de.tu_dresden.lat.prettyPrinting.formatting.SimpleOWLFormatter
import de.tu_dresden.lat.proofs.interfaces.{IInference, IProof, IProofWriter}
import de.tu_dresden.lat.proofs.json.JsonProofWriter
import org.semanticweb.owlapi.model.parameters.Imports

import scala.collection.JavaConverters._
import scala.collection.mutable

class Proof2LISPFormatter(ontology: OWLOntology) extends IProofWriter {

  val axiomFormatter = new ELAxiom2LISPFormatter(ontology.getOWLOntologyManager().getOWLDataFactory())
  val conceptFormatter = axiomFormatter.conceptFormatter

 // var start = new Date().getTime

 // var count = 0

  override def writeToFile(proof: IProof, s: String): Unit = {
    val writer = new PrintWriter(s+".lisp")
    writer.write(toString(proof));
    writer.close();
  }

  override def writeToFile(proofs: util.Collection[IProof], s: String): Unit = {
    val writer = new PrintWriter(s+".lisp")
    proofs.asScala.foreach(p => writer.write(toString(p)))
    writer.close()
  }

  var bracketsToClose: Int = 0

  override def toString(proof: IProof): String = {

   // count+=1
   // println(count)

   /* if((new Date().getTime-start)>60000)
      {
        println("Time out!")
        System.exit(1)
      }
*/
     bracketsToClose = 2

    cleanCash()

      "(check \n"+
      generateRoleSignature(ProofTools.objectPropertiesInSignature(proof))+
      generateConceptSignature(ProofTools.classesInSignature(proof))+
      "(% g (goal "+proof.getFinalConclusion().accept(axiomFormatter)+")\n\n"+
      makeAssumptions(ProofTools.assertedAxiomsUsed(proof))+
      printInferences(proof)+
      printClosingBrackets()+
    "\n\n"
  }

  var axiomMap = Map[OWLAxiom, Int]()

  def axiomID(axiom: OWLAxiom) = {
    "p"+axiomMap(axiom)
  }

  def cleanCash() =
    knownSubProofs = new util.HashMap[OWLAxiom, String]()

  private def printInferences(proof: IProof) = {

    "(: done (Proved _ g \n"+
    printInference(proof.getFinalConclusion, proof, Set[IInference]() )+
    "))"
  }

  var knownSubProofs = new util.HashMap[OWLAxiom, String]()
  var reflexiveMap = Map[OWLAxiom, OWLAxiom]() // to catch errors in the Map


  /**
   *
   * @param axiom
   * @param proof
   * @param visited keeps track of the axioms visited on the current branch of the proof, to avoid cycles: only follow inferences that are not based on things we are proving!
   * @return
   */
  private def printInference( axiom: OWLAxiom,
                              proof: IProof,
                              visited: Set[IInference]): String = {

    // the reflexive map is needed in some rare cases, where an axiom is for some reason wrongly mapped
    if(knownSubProofs.containsKey(axiom) && reflexiveMap(axiom).equals(axiom)) {
      //println("Reuse!")
      return knownSubProofs.get(axiom)
    }


    val inference: IInference =
      proof.getInferences(axiom).iterator().asScala.find(!visited(_)) match {
        case Some(value) => value
        case None =>
          throw new RuntimeException("cyclic inference!")
      }

   // println("Current inference: "+inference)

    val newVisited = visited + inference

    val result = try {
      inference.getRuleName match {
        case "Asserted Conclusion" =>
   //       println("For "+axiom)
   //       println("which is "+axiomMap(axiom))
          axiomID(axiom)
        case "Top Superclass" =>
          "(Top " +
            inference.getConclusion().asInstanceOf[OWLSubClassOfAxiom].getSubClass.accept(conceptFormatter) +
            " )\n"
        case "Bottom Subclass" =>
          "(Bottom " +
            inference.getConclusion().asInstanceOf[OWLSubClassOfAxiom].getSuperClass.accept(conceptFormatter) +
            ")\n"
        case "Class Inclusion Tautology" => "(subRef " +
          inference.getConclusion().asInstanceOf[OWLSubClassOfAxiom].getSubClass.accept(conceptFormatter) +
          " )\n"
        case "Equivalent Classes Decomposition" if 
	  inference
	    .getConclusion()
	    .asInstanceOf[OWLSubClassOfAxiom]
	    .getSubClass
	    .equals(
		inference
		  .getPremises
		  .get(0)
		  .asInstanceOf[OWLEquivalentClassesAxiom]
		  .getClassExpressionsAsList()
		  .get(0)
	    ) =>
	  "(EqDec1 _ _ "+printInference(inference.getPremises.get(0),proof,newVisited) +
	  ")\n"
        case "Equivalent Classes Decomposition" if 
	  inference
	    .getConclusion()
	    .asInstanceOf[OWLSubClassOfAxiom]
	    .getSubClass
	    .equals(
		inference
		  .getPremises
		  .get(0)
		  .asInstanceOf[OWLEquivalentClassesAxiom]
		  .getClassExpressionsAsList()
		  .get(1)
	    ) =>
	  "(EqDec2 _ _ "+printInference(inference.getPremises.get(0),proof,newVisited)+
	  ")\n"
	
        case "Class Hierarchy" =>
          assert(inference.getPremises.size == 2)
          "(ConHi _ _ _ " + printInference(inference.getPremises.get(0), proof, newVisited) +
            " " +
            printInference(inference.getPremises.get(1), proof, newVisited) +
            ")\n"
        case "Class Inclusion Cycle" => {
          assert(inference.getPremises.size == 2)
          assert(!inference.getPremises.get(0).equals(inference.getPremises.get(1)))
   //       println(inference.getPremises)
          if (inference.getPremises.size != 2) {
            println(inference)
            println("not 2 presmises")
            System.exit(1);
          }
          val sorted = inference.getPremises //sortPremises(inference)
      //    println(axiomMap)
//          println("(ConCyc _ _ " + printInference(inference.getPremises.get(0), proof, newVisited) +
//            " " +
//           printInference(inference.getPremises.get(1), proof, newVisited) +
//            ")\n")
          "(ConCyc _ _ " + printInference(inference.getPremises.get(0), proof, newVisited) +
            " " +
            printInference(inference.getPremises.get(1), proof, newVisited) +
            ")\n"
        }
        case "Intersection Decomposition" =>
          val ci = inference.getConclusion.asInstanceOf[OWLSubClassOfAxiom]
          val lhs = ci.getSubClass.asInstanceOf[OWLObjectIntersectionOf]
          val rhs = ci.getSuperClass
          "(InterDec "+lhs.accept(conceptFormatter)+" "+rhs.accept(conceptFormatter)+")" // support arbitrary? let's see...
        case "Intersection Composition" =>
          val lhs = axiom.asInstanceOf[OWLSubClassOfAxiom].getSubClass
          def recursiveComposition(premises: List[OWLAxiom]): String = premises match {
            case first :: (second :: Nil) =>
              "(InterComp _ _ _ " +
                printInference(first, proof, newVisited) +
                " " +
                printInference(second, proof, newVisited) +
                ")\n"
            case first :: rest =>
              "(InterComp _ _ _ " +
                printInference(first, proof, newVisited) +
                recursiveComposition(rest) +
              ")\n"
          }
          recursiveComposition(inference.getPremises().asScala.toList)

         /*    non-recursive case with only 2 premises:
          recursiveIntersectionComposition(inference, proof, newVisited)
          "(InterComp _ _ _ " +
            printInference(inference.getPremises.get(0), proof, newVisited) +
            " " +
            printInference(inference.getPremises.get(1), proof, newVisited) +
            ")\n"
          */
        case "Existential Filler Expansion" =>
          assert(inference.getPremises.size == 1)
          "(ExExpand _ _ _ " +
            printInference(inference.getPremises.get(0), proof, newVisited) +
            ")\n"
        case "Existential Property Expansion" =>
          assert(inference.getPremises.size == 1)
          "(ExExpandRole " +
            inference.getConclusion
              .asInstanceOf[OWLSubClassOfAxiom]
              .getSubClass
              .asInstanceOf[OWLObjectSomeValuesFrom]
              .getFiller
              .accept(conceptFormatter) +
            " _ _ " +
            printInference(inference.getPremises.get(0), proof, newVisited) +
            ")\n"
        case "Existential Composition" if inference.getPremises.size == 3 =>
          "(ExComp _ _ _ _ _ _ _ " +
            inference.getPremises.asScala.map(printInference(_, proof, newVisited)).mkString(" ") +
            ")\n"
	case "Existential Composition" if inference.getPremises.size>3 => 
	  nAryExistentialComposition(inference, proof, newVisited)
        case "Property Inclusion Tautology" =>
          assert(inference.getPremises.size == 0)
          "(rsubRef " +
            conceptFormatter.formatRole(
              inference.getConclusion
                .asInstanceOf[OWLSubObjectPropertyOfAxiom]
                .getSubProperty
            )+
            ")\n"
        case "Property Hierarchy" =>
          assert(inference.getPremises.size == 2)
          "(RolHi _ _ _ " +
            inference.getPremises.asScala.map(printInference(_, proof, newVisited)).mkString(" ") +
            ")\n"
        case "Property Domain Transaltion" => // this is the spelling produced by ELK for this rule
          assert(inference.getPremises.size==1)
          "(RoleDomain _ _ " +
          printInference(inference.getPremises.get(0), proof, newVisited) +
          ")\n"
        case "Transitive Property Translation" =>
          assert(inference.getPremises.size==1)
          "(TransitiveRole _ _ "+printInference(inference.getPremises.get(0), proof, newVisited) +
          ")\n"
        case "Negation Clash" =>
          assert(inference.getPremises.size==0)
          "(NegationClash "+
            inference.getConclusion
              .asInstanceOf[OWLSubClassOfAxiom]
              .getSubClass
              .asInstanceOf[OWLObjectIntersectionOf]
              .getOperandsAsList
              .get(0)
              .accept(conceptFormatter)+
          ")"
        case _ =>
          throw new MatchError(inference)
      }
    } catch {
      case re: RuntimeException if re.getMessage=="cyclic inference!" => {
        //println("Current line of inference is cyclic -> back tracking!")

        printInference(axiom, proof, newVisited) // exclude decision point chosen next time
      }
    }

    // the reflexive map is for double checking that the values in the map correspond to the axiom
    knownSubProofs.put(axiom, result)
    reflexiveMap += (axiom -> axiom)

    result
  }

  private def nAryExistentialComposition(
    inference: IInference, 
    proof: IProof, 
    newVisited: Set[IInference]): String = {
    
    val premises = inference.getPremises().asScala.toList

    def inner(premisesLeft: List[OWLAxiom]): String = 
      premisesLeft match {
	case Nil => assert(false);"" // this should never happen
	case secondLast:: (_ :: Nil) => 
	  // the last one is the role axiom dealt with below
	  printInference(secondLast, proof, newVisited)
	case nextPremise :: restPremises => 
	  "(ExComp1 _ _ _ _ _ _ "+
	    printInference(nextPremise, proof, newVisited)+ 
	    " " +
	    inner(restPremises)+
	    ")"	  
      }
  

    "(ExComp2 _ _ _ _ "+
      inner(premises)+
      " "+
      printInference(premises.last, proof, newVisited)+
    ")"
    
  }

  private def sortPremises(inference: IInference): List[OWLAxiom] = inference.getRuleName match {
    case "Class Inclusion Cycle" => {

      //(! p1 (holds (sub c d))
      //(! p2 (holds (sub d c))
      //; -----------------------
      //(holds (eq c d))
      inference.getConclusion() match {
        case eq : OWLEquivalentClassesAxiom =>
          val firstC = eq.getClassExpressionsAsList.get(0)
          val secondC = eq.getClassExpressionsMinus(firstC)

          assert(inference.getPremises().size()==2, inference.toString)

          if(inference.getPremises().get(0).asInstanceOf[OWLSubClassOfAxiom].getSubClass.equals(firstC))
            return inference.getPremises().asScala.toList
          else
            return List(inference.getPremises().get(1), inference.getPremises().get(0))
      }
    }
  }

  private def makeAssumptions(assumptions: Set[OWLAxiom]) = {
    var count = 0

    axiomMap = Map()

    "; assumptions (axioms in the ontology)\n"+
    assumptions.map { ax =>
      count += 1

      val before = axiomMap.size

      axiomMap += ((ax -> count))

      val after = axiomMap.size

      if(before==after){
        println(axiomMap)
        println("something wrong with map")
        System.exit(1)
      }

      bracketsToClose+=1

      val axiomString = ax.accept(axiomFormatter)

      assert(axiomString!=null, "Problem with: "+ax)

      "(% p" + count + " (holds " + axiomString + ")"
    }.mkString("\n")+"\n"
  }


  private def generateRoleSignature(roles: Iterable[OWLObjectProperty]) =
    "; role signature \n" +
    roles.map { role =>
      bracketsToClose += 1
      "(% " + conceptFormatter.formatRoleName(role) + " rname "
    }.mkString("\n")+"\n\n"

  private def generateConceptSignature(concepts: Iterable[OWLClass]) = {
    var count = 0
    "; concept signature \n" +
      concepts.map{concept =>
      count+=1
        bracketsToClose+=1
      "(@ "+conceptFormatter.formatConceptName(concept)+" (ac "+count+")"
      }.mkString("\n")+"\n\n"
  }

  private def printClosingBrackets() =
    ")"* bracketsToClose
}
