package de.tu_dresden.lat.dlProofChecking.proofsProcessing

import java.util.Date

import de.tu_dresden.lat.proofs.interfaces.{IProof, IProofGenerator}
import org.semanticweb.owlapi.apibinding.OWLManager
import org.semanticweb.owlapi.model.{OWLClass, OWLOntology}
import org.semanticweb.owlapi.reasoner.{InferenceType, Node, OWLReasoner, OWLReasonerFactory}

import scala.collection.mutable

abstract class ProofsProcessor(reasonerFactory: OWLReasonerFactory, proofGenerator: IProofGenerator) {

  val dataFactory = OWLManager.getOWLDataFactory

  var classificationSize: Int = _

  def processOntology(ontology: OWLOntology): Unit = {
    val reasoner =  reasonerFactory.createReasoner(ontology)

    var start = new Date().getTime()

    reasoner.precomputeInferences(InferenceType.CLASS_HIERARCHY)

    println("Reasoning time: "+(new Date().getTime()-start))

    proofGenerator.setReasoner(reasoner)

    //val proofGenerator = new ELKProofGenerator(reasoner)

    classificationSize = 0

    start = new Date().getTime()



    process(reasoner.getTopClassNode,
      reasoner,
      proofGenerator,
      mutable.Set[Node[OWLClass]](),
      ontology)

    println("Proof-processing time: "+(new Date().getTime()-start))

    println("ClassificationSize: "+classificationSize)
  }

  protected def check(proof: IProof): Unit

  private def process(node: Node[OWLClass],
                      reasoner: OWLReasoner,
                      proofGenerator: IProofGenerator,
                      processed: mutable.Set[Node[OWLClass]],
                      ontology: OWLOntology): Any = {

//    if(classificationSize>=5) {
//      println("Done for today.")
//      System.exit(1)
//    }

    if(processed(node))
      return

    processed.add(node)

    val rep = node.getRepresentativeElement
    node.getEntitiesMinus(rep).forEach(proofForEq(rep, _, proofGenerator, ontology))


    reasoner.getSubClasses(rep,true).forEach { subClassNode =>
      if(!subClassNode.getRepresentativeElement.isBottomEntity && !rep.isTopEntity) {
        proofForSub(subClassNode.getRepresentativeElement, rep, proofGenerator, ontology)
      }
      process(subClassNode, reasoner, proofGenerator, processed, ontology)
    }
  }

  private def proofForEq(class1: OWLClass, class2: OWLClass,
                         proofGenerator: IProofGenerator, ontology: OWLOntology): Unit =
    if(!ontology.containsAxiom(dataFactory.getOWLEquivalentClassesAxiom(class1, class2))) {
      classificationSize += 1
      println(classificationSize)
      val proof = proofGenerator.proveEquivalence(class1, class2);
      check(proof)
    }

  private def proofForSub(subClass: OWLClass, superClass: OWLClass,
                          proofGenerator: IProofGenerator, ontology: OWLOntology) =
    if(!ontology.containsAxiom(dataFactory.getOWLSubClassOfAxiom(subClass, superClass))) {
      classificationSize += 1
      println(classificationSize)
      val proof = proofGenerator.proveSubsumption(subClass, superClass)
      check(proof)
    }
}
