package de.tu_dresden.lat.dlProofChecking.tools

import java.util

import com.clarkparsia.owlapi.explanation.{BlackBoxExplanation, SingleExplanationGenerator}
import de.tu_dresden.lat.prettyPrinting.formatting.SimpleOWLFormatter
import de.tu_dresden.lat.proofs.interfaces.{IProof, IProofGenerator}
import org.liveontologies.puli.{DynamicProof, InferenceJustifier, InferenceJustifiers}
import org.liveontologies.puli.pinpointing.{InterruptMonitor, MinimalSubsetCollector, MinimalSubsetEnumerator, MinimalSubsetEnumerators}
import org.semanticweb.elk.owl.inferences.ElkProofGenerator
import org.semanticweb.elk.owlapi.ElkReasoner
import org.semanticweb.elk.owlapi.proofs.{ElkOwlInference, ElkOwlProof}
import org.semanticweb.owlapi.apibinding.OWLManager
import org.semanticweb.owlapi.model.{OWLAxiom, OWLClass, OWLClassExpression, OWLObjectComplementOf, OWLOntology, OWLOntologyManager, OWLSubClassOfAxiom}
import org.semanticweb.owlapi.reasoner.{OWLReasoner, OWLReasonerFactory}

/**
 * Uses the given proof generator to generate proofs, but only on a justification for the respective entailment.
 *
 * Note: reduces entailment to concept satisfiability, and thus requires a reasoner that can deal with
 * disjunction and negation.
 *
 * @param delegate
 */
class ProofWithJustificationGenerator(delegate: IProofGenerator) extends IProofGenerator {

  var ontology: OWLOntology = _
  var reasoner: OWLReasoner = _
  var reasonerFactory: OWLReasonerFactory = _

  var dataFactory = OWLManager.getOWLDataFactory
  var ontologyManager: OWLOntologyManager = _

  var explanationGenerator: SingleExplanationGenerator = _

  var changed=true

  override def setOntology(ontology: OWLOntology): Unit = {
    this.ontology=ontology
    this.ontologyManager=ontology.getOWLOntologyManager
    changed=true
  }

  def setReasonerFactory(reasonerFactory: OWLReasonerFactory) = {
    this.reasonerFactory=reasonerFactory
    changed=true
  }

  override def setReasoner(reasoner: OWLReasoner): Unit = {
    this.reasoner=reasoner
    changed=true
  }

  override def proveSubsumption(owlClass: OWLClass, owlClass1: OWLClass): IProof = {
    val axiom = dataFactory.getOWLSubClassOfAxiom(owlClass, owlClass1)
    val justification = getJustification(axiom);
    delegate.setOntology(justification)
    delegate.proveSubsumption(owlClass,owlClass1)
  }

  override def proveEquivalence(owlClass: OWLClass, owlClass1: OWLClass): IProof = {
    val axiom = dataFactory.getOWLEquivalentClassesAxiom(owlClass, owlClass1)
    val justification = getJustification(axiom)


    println("justification:")
    println(SimpleOWLFormatter.format(justification))

    delegate.setOntology(justification)
    delegate.proveEquivalence(owlClass,owlClass1)
  }


  def getJustification(axiom: OWLAxiom) = {
 /*   val proof = ElkOwlProof.create(reasoner.asInstanceOf[ElkReasoner], axiom)

    //final InferenceJustifier<org.liveontologies.puli.Inference<OWLAxiom>, ? extends Set<? extends OWLAxiom>> justifier = InferenceJustifiers

    val justifier =
      InferenceJustifiers.justifyAssertedInferences[OWLAxiom]



    val collector = new MinimalSubsetCollector[OWLAxiom]()


    MinimalSubsetEnumerators.enumerateJustifications(axiom, proof, justifier, InterruptMonitor.DUMMY, collector);

    var just = collector.getSets.iterator().next()
*/
    val ontology = ontologyManager.createOntology()

//    just.forEach(ontology.add)

    ontology
  }

}
