package de.tu_dresden.lat.dlProofChecking.tools

import java.io.File

import de.tu_dresden.lat.dlProofChecking.proofsProcessing.ProofsProcessor
import org.semanticweb.owlapi.model.OWLClass
import org.semanticweb.owlapi.model.OWLOntology
import org.semanticweb.owlapi.reasoner.{InferenceType, Node, OWLReasoner}
import de.tu_dresden.lat.proofs.ELKProofGenerator
import de.tu_dresden.lat.proofs.interfaces.{IInference, IProof, IProofGenerator}
import org.semanticweb.elk.owlapi.ElkReasonerFactory
import org.semanticweb.owlapi.apibinding.OWLManager

import scala.collection.mutable


/**
 * Find all rules used in a corpus of ontologies.
 */
object RuleFinder extends ProofsProcessor(new ElkReasonerFactory(), new ELKProofGenerator()) {

  var knownRules = Set[String]()

  def main(args: Array[String]): Unit = {
    val manager = OWLManager.createOWLOntologyManager();

//    val dir = new File(args(0))

//    dir.listFiles().foreach { file =>

    val file = new File(args(0))

      if(file.getName.endsWith(".owl")) {
        println(file+"...")
        val ontology = manager.loadOntologyFromOntologyDocument(file)
        println("processing...")
        processOntology(ontology)
      }
    //}

  }



  override def check(proof: IProof) = {
    //println(proof)

    proof.getInferences().forEach(inf =>
      if (!knownRules(inf.getRuleName)) {
        println("New rule: " + inf.getRuleName)
        println("Inference: ")
        println(inf);
        knownRules += inf.getRuleName
      }
    )
  }
}
